@@ -1,18 +1,13 @@
-# pyyaml/lib/yaml/__init__.py
-# pyyaml/lib/yaml/loader.py
-# pyyaml/lib/yaml/dumper.py
-
 package YAML::Perl;
 use 5.005003;
 use strict;
 use warnings; # XXX requires 5.6+
-use Carp;
 use YAML::Perl::Base -base;
 
-$YAML::Perl::VERSION = '0.01';
+$YAML::Perl::VERSION = '0.02';
 
 @YAML::Perl::EXPORT = qw'Dump Load';
-@YAML::Perl::EXPORT_OK = qw'DumpFile LoadFile freeze thaw';
+@YAML::Perl::EXPORT_OK = qw'DumpFile LoadFile freeze thaw emit';
 
 field dumper_class => -chain,
     -class => '-init',
@@ -36,11 +31,17 @@ field resolver =>
     -init => '$self->create("resolver")';
 
 sub Dump {
-    return YAML::Perl->new->dumper->dump(@_);
+    my $dumper = YAML::Perl->new()->dumper;
+    $dumper->open();
+    $dumper->dump(@_);
+    $dumper->close;
+    return $dumper->stream;
 }
 
 sub Load {
-    return YAML::Perl->new->loader->load(@_);
+    my $loader = YAML::Perl->new->loader;
+    $loader->open(@_);
+    return ($loader->load());
 }
 
 {
@@ -63,7 +64,7 @@ sub DumpFile {
         open $OUT, $mode, $filename
           or YAML::Perl::Base->die('YAML_DUMP_ERR_FILE_OUTPUT', $filename, $!);
     }  
-    local $/ = "\n"; # reset special to "sane"
+    local $/ = "\n"; # rset special to "sane"
     print $OUT Dump(@_);
 }
 
@@ -80,41 +81,18 @@ sub LoadFile {
     return Load(do { local $/; <$IN> });
 }
 
+# Ported EXPORT_OK top level functions from PyYaml
+
+sub emit {
+    my $events = shift;
+    require YAML::Perl::Emitter;
+    my $emitter = YAML::Perl::Emitter->new->open;
+    for my $event (@$events) {
+        $emitter->emit($event);
+    }
+    my $buffer = $emitter->writer->stream->buffer;
+    $emitter->close;
+    return $$buffer;
+}
+
 1;
-
-=encoding utf8
-
-=head1 NAME
-
-YAML::Perl - Pure Perl Port of PyYAML
-
-=head1 WARNING
-
-This is a very early release. Don't even bother to try this version.
-
-=head1 SYNOPSIS
-
-    use YAML::Perl;
-
-    my %inc = Load Dump %INC;
-
-=head1 DESCRIPTION
-
-PyYAML is the most robust and correct YAML module for a dynamic
-language. It is (obviously) written in/for Python. This module is a
-complete port of PyYAML to Perl.
-
-=head1 AUTHOR
-
-Ingy döt Net <ingy@cpan.org>
-
-=head1 COPYRIGHT
-
-Copyright (c) 2008. Ingy döt Net.
-
-This program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
-
-See http://www.perl.com/perl/misc/Artistic.html
-
-=cut
