@@ -1,5 +1,3 @@
-# pyyaml/lib/yaml/nodes.py
-
 package YAML::Perl::Nodes;
 use strict;
 use warnings;
@@ -13,12 +11,13 @@ field 'tag';
 field 'value';
 field 'start_mark';
 field 'end_mark';
+field 'style';
 
 sub stringify {
     my $self = shift;
     my $class = ref($self) || $self;
     my $value = $self->value . "";
-    my $tag = $self->tag . "";
+    my $tag = ($self->tag || '') . "";
     return "$class(tag=$tag, value=$value)";
 }
 
