@@ -102,14 +102,11 @@ sub parse {
         return @events;
     }
     else {
-        return sub {
-            return $self->check_event() ? $self->get_event() : undef;
-        };
+        return $self->check_event() ? $self->get_event() : ();
     }
 }
 
 sub check_event {
-    # print "+check_event\n";
     my $self = shift;
     my @choices = @_;
     if (not defined $self->current_event) {
@@ -132,7 +129,6 @@ sub check_event {
 }
 
 sub peek_event {
-    # print "+peek_event\n";
     my $self = shift;
     if (not defined $self->current_event) {
         if (my $state = $self->state) {
@@ -143,11 +139,7 @@ sub peek_event {
 }
 
 sub get_event {
-    # print "+get_event\n";
     my $self = shift;
-    # print $self->state . "\n";
-    # print "  " . $self->scanner->peek_token() . "\n";
-    # print "  [@{$self->scanner->tokens}]\n";
     if (not defined $self->current_event) {
         if (my $state = $self->state) {
             $self->current_event($self->$state());
@@ -155,7 +147,6 @@ sub get_event {
     }
     my $value = $self->current_event;
     $self->current_event(undef);
-    # print "    $value\n";
     return $value;
 }
 
@@ -264,7 +255,10 @@ sub parse_document_end {
 sub parse_document_content {
     my $self = shift;
     if ( $self->scanner->check_token( 
-        map "YAML::Perl::Token::${_}", qw/ DocumentStart DocumentEnd StreamEnd / 
+        'YAML::Perl::Token::Directive',
+        'YAML::Perl::Token::DocumentStart',
+        'YAML::Perl::Token::DocumentEnd',
+        'YAML::Perl::Token::StreamEnd',
     ) ) {
         my $event = $self->process_empty_scalar( $self->scanner->peek_token()->start_mark() );
         $self->state( pop @{ $self->states() } );
@@ -277,11 +271,50 @@ sub parse_document_content {
 
 sub process_directives {
     my $self = shift;
-    $self->{yaml_version} = undef;
-    $self->{tag_handles} = {};
+    $self->yaml_version(undef);
+    $self->tag_handles({});
     while ($self->scanner->check_token('YAML::Perl::Token::Directive')) {
-        XXX my $token = $self->get_token();
+        my $token = $self->scanner->get_token();
+        if ($token->name eq 'YAML') {
+            if (defined($self->yaml_version)) {
+                throw YAML::Perl::Error::Parser(
+                    "found duplicate YAML directive", $token->start_mark);
+            }
+            my ($major, $minor) = split('\.', $token->value);
+            if ($major != 1) {
+                throw YAML::Perl::Error::Parser(
+                    "found incompatible YAML document (version 1.* is required)",
+                    $token->start_mark);
+            }
+            $self->yaml_version($token->value);
+        }
+        elsif ($token->name eq 'TAG') {
+            my ($handle, $prefix) = @{$token->value};
+            if (defined $self->tag_handles->{$handle}) {
+                throw YAML::Perl::Error::Parser(
+                    undef,
+                    undef,
+                    "duplicate tag handle %r",
+                    $handle->encode('utf-8'),
+                    $token->start_mark,
+                );
+            }
+            $self->tag_handles->{$handle} = $prefix;
+        }
     }
+    my @value;
+    if (keys(%{$self->tag_handles})) {
+        @value = ($self->yaml_version, {%{$self->tag_handles}});
+    }
+    else {
+        @value = ($self->yaml_version, undef);
+    }
+    for my $key (keys %{$self->DEFAULT_TAGS}) {
+        if (not exists $self->tag_handles->{$key}) {
+            $self->tag_handles->{$key} = $self->DEFAULT_TAGS->{$key};
+        }
+    }
+    return @value;
 }
 
 sub parse_block_node {
@@ -305,7 +338,7 @@ sub parse_node {
     
     my $event;
     if ($self->scanner->check_token('YAML::Perl::Token::Alias')) {
-        my $token = $self->get_token();
+        my $token = $self->scanner->get_token();
         $event = YAML::Perl::Event::Alias->new(
             anchor     => $token->value,
             start_mark => $token->start_mark,
@@ -378,7 +411,7 @@ sub parse_node {
             if ($self->scanner->check_token('YAML::Perl::Token::Scalar')) {
                 my $token = $self->scanner->get_token();
                 $end_mark = $token->end_mark;
-                if (($token->plain and not defined $tag) or $tag eq '!') {
+                if (($token->plain and not defined $tag) or ($tag || '') eq '!') {
                     $implicit = [True, False];
                 }
                 elsif (not defined $tag) {
@@ -420,7 +453,7 @@ sub parse_node {
                     end_mark => $end_mark,
                     flow_style => True,
                 );
-                $self->state('parse_flow_mapping_first_entry');
+                $self->state('parse_flow_mapping_first_key');
             }
             elsif ($self->scanner->check_token('YAML::Perl::Token::BlockSequenceStart')) {
                 $end_mark = $self->scanner->peek_token()->end_mark;
@@ -478,12 +511,63 @@ sub parse_block_sequence_first_entry {
 
 sub parse_block_sequence_entry {
     my $self = shift;
-    die "parse_block_sequence_entry";
+    if ($self->scanner->check_token('YAML::Perl::Token::BlockEntry')) {
+        my $token = $self->scanner->get_token();
+        if (not $self->scanner->check_token(qw(
+            YAML::Perl::Token::BlockEntry
+            YAML::Perl::Token::BlockEnd
+        ))) {
+            push @{$self->states}, 'parse_block_sequence_entry';
+            return $self->parse_block_node();
+        }
+        else {
+            $self->state('parse_block_sequence_entry');
+            return $self->process_empty_scalar($token->end_mark);
+        }
+    }
+    if (not $self->scanner->check_token('YAML::Perl::Token::BlockEnd')) {
+        my $token = $self->scanner->peek_token();
+        throw YAML::Perl::Error::Parser(
+            "while parsing a block collection", $self->marks->[-1],
+            "expected <block end>, but found ", $token->id, $token->start_mark
+        );
+    }
+    my $token = $self->scanner->get_token();
+    my $event = YAML::Perl::Event::SequenceEnd->new(
+        start_mark => $token->start_mark,
+        end_mark => $token->end_mark,
+    );
+    $self->state(pop @{$self->states});
+    pop @{$self->marks};
+    return $event;
 }
 
 sub parse_indentless_sequence_entry {
     my $self = shift;
-    die "parse_indentless_sequence_entry";
+    my $token;
+    if ($self->scanner->check_token('YAML::Perl::Token::BlockEntry')) {
+        $token = $self->scanner->get_token();
+        if (not $self->scanner->check_token(
+                'YAML::Perl::Token::BlockEntry',
+                'YAML::Perl::Token::Key',
+                'YAML::Perl::Token::Value',
+                'YAML::Perl::Token::BlockEnd',
+            )) {
+            push @{$self->states}, 'parse_indentless_sequence_entry';
+            return $self->parse_block_node();
+        }
+        else {
+            $self->state('parse_indentless_sequence_entry');
+            return $self->process_empty_scalar($token->end_mark);
+        }
+    }
+    $token = $self->scanner->peek_token();
+    my $event = YAML::Perl::Event::SequenceEnd->new(
+        start_mark => $token->start_mark,
+        end_mark => $token->end_mark,
+    );
+    $self->state(pop @{$self->states});
+    return $event;
 }
 
 sub parse_block_mapping_first_key {
@@ -514,7 +598,7 @@ sub parse_block_mapping_key {
         my $token = $self->scanner->peek_token();
         throw YAML::Perl::Error::Parser(
             "while parsing a block mapping", $self->marks->[-1],
-            "expected <block end>, but found %r", $token->id, $token->start_mark
+            "expected <block end>, but found ", $token->id, $token->start_mark
         );
     }
     my $token = $self->scanner->get_token();
@@ -553,47 +637,194 @@ sub parse_block_mapping_value {
 
 sub parse_flow_sequence_first_entry {
     my $self = shift;
-    die "parse_flow_sequence_first_entry";
+    my $token = $self->scanner->get_token();
+    push @{$self->marks}, $token->start_mark;
+    return $self->parse_flow_sequence_entry(True);
 }
 
 sub parse_flow_sequence_entry {
     my $self = shift;
-    die "parse_flow_sequence_entry";
+    my $first = @_ ? shift : False;
+    if (not $self->scanner->check_token('YAML::Perl::Token::FlowSequenceEnd')) {
+        if (not $first) {
+            if ($self->scanner->check_token('YAML::Perl::Token::FlowEntry')) {
+                $self->scanner->get_token();
+            }
+            else {
+                my $token = $self->scanner->peek_token();
+                throw YAML::Perl::Error::Parser(
+                    "while parsing a flow sequence",
+                    $self->marks->[-1],
+                    "expected ',' or ']', but got %r",
+                    $token->id,
+                    $token->start_mark
+                );
+            }
+        }
+        
+        if ($self->scanner->check_token('YAML::Perl::Token::Key')) {
+            my $token = $self->scanner->peek_token();
+            my $event = YAML::Perl::Event::MappingStart->new(
+                anchor => undef,
+                tag => undef,
+                implicit => True,
+                start_mark => $token->start_mark,
+                end_mark => $token->end_mark,
+                flow_style => True,
+            );
+            $self->state('parse_flow_sequence_entry_mapping_key');
+            return $event;
+        }
+        elsif (not $self->scanner->check_token('YAML::Perl::Token::FlowSequenceEnd')) {
+            push @{$self->states}, 'parse_flow_sequence_entry';
+            return $self->parse_flow_node();
+        }
+    }
+    my $token = $self->scanner->get_token();
+    my $event = YAML::Perl::Event::SequenceEnd->new(
+        start_mark => $token->start_mark,
+        end_mark => $token->end_mark,
+    );
+    $self->state(pop @{$self->states});
+    pop @{$self->marks};
+    return $event;
 }
 
 sub parse_flow_sequence_entry_mapping_key {
     my $self = shift;
-    die "parse_flow_sequence_entry_mapping_key";
+    my $token = $self->scanner->get_token();
+    if (not $self->scanner->check_token(
+        'YAML::Perl::Token::Value',
+        'YAML::Perl::Token::FlowEntry',
+        'YAML::Perl::Token::FlowSequenceEnd',
+    )) {
+        push @{$self->states}, 'parse_flow_sequence_entry_mapping_value';
+        return $self->parse_flow_node();
+    }
+    else {
+        $self->state('parse_flow_sequence_entry_mapping_value');
+        return $self->process_empty_scalar($token->end_mark);
+    }
 }
 
 sub parse_flow_sequence_entry_mapping_value {
     my $self = shift;
-    die "parse_flow_sequence_entry_mapping_value";
+    if ($self->scanner->check_token('YAML::Perl::Token::Value')) {
+        my $token = $self->scanner->get_token();
+        if (not $self->scanner->check_token(
+            'YAML::Perl::Token::FlowEntry',
+            'YAML::Perl::Token::FlowSequenceEnd'
+        )) {
+            push @{$self->states}, 'parse_flow_sequence_entry_mapping_end';
+            return $self->parse_flow_node();
+        }
+        else {
+            $self->state('parse_flow_sequence_entry_mapping_end');
+            return $self->process_empty_scalar($token->end_mark);
+        }
+    }
+    else {
+        $self->state('parse_flow_sequence_entry_mapping_end');
+        my $token = $self->scanner->peek_token();
+        return $self->process_empty_scalar($token->start_mark);
+    }
 }
 
 sub parse_flow_sequence_entry_mapping_end {
     my $self = shift;
-    die "parse_flow_sequence_entry_mapping_end";
+    $self->state('parse_flow_sequence_entry');
+    my $token = $self->scanner->peek_token();
+    return YAML::Perl::Event::MappingEnd->new(
+        start_mark => $token->start_mark,
+        end_mark => $token->start_mark,
+    );
 }
 
 sub parse_flow_mapping_first_key {
     my $self = shift;
-    die "parse_flow_mapping_first_key";
+    my $token = $self->scanner->get_token();
+    push @{$self->marks}, $token->start_mark;
+    return $self->parse_flow_mapping_key(True)
 }
 
 sub parse_flow_mapping_key {
     my $self = shift;
-    die "parse_flow_mapping_key";
+    my $first = @_ ? shift : False;
+
+    if (not $self->scanner->check_token('YAML::Perl::Token::FlowMappingEnd')) {
+        if (not $first) {
+            if ($self->scanner->check_token('YAML::Perl::Token::FlowEntry')) {
+                $self->scanner->get_token();
+            }
+            else {
+                my $token = $self->scanner->peek_token();
+                throw YAML::Perl::Error::Parser(
+                    "while parsing a flow mapping",
+                    $self->marks->[-1],
+                    "expected ',' or '}', but got %r",
+                    $token->id,
+                    $token->start_mark
+                );
+            }
+        }
+        if ($self->scanner->check_token('YAML::Perl::Token::Key')) {
+            my $token = $self->scanner->get_token();
+            if (not $self->scanner->check_token(
+                'YAML::Perl::Token::Value',
+                'YAML::Perl::Token::FlowEntry',
+                'YAML::Perl::Token::FlowMappingEnd',
+            )) {
+                push @{$self->states}, 'parse_flow_mapping_value';
+                return $self->parse_flow_node();
+            }
+            else {
+                $self->state('parse_flow_mapping_value');
+                return $self->process_empty_scalar($token->end_mark);
+            }
+        }
+        elsif (not $self->scanner->check_token('YAML::Perl::Token::FlowMappingEnd')) {
+            push @{$self->states}, 'parse_flow_mapping_empty_value';
+            return $self->parse_flow_node();
+        }
+    }
+    my $token = $self->scanner->get_token();
+    my $event = YAML::Perl::Event::MappingEnd->new(
+        start_mark => $token->start_mark,
+        end_mark => $token->end_mark,
+    );
+    $self->state(pop @{$self->states});
+    pop @{$self->marks};
+    return $event;
 }
 
 sub parse_flow_mapping_value {
     my $self = shift;
-    die "parse_flow_mapping_value";
+
+    if ($self->scanner->check_token('YAML::Perl::Token::Value')) {
+        my $token = $self->scanner->get_token();
+        if (not $self->scanner->check_token(
+            'YAML::Perl::Token::FlowEntry',
+            'YAML::Perl::Token::FlowMappingEnd',
+        )) {
+            push @{$self->states}, 'parse_flow_mapping_key';
+            return $self->parse_flow_node();
+        }
+        else {
+            $self->state('parse_flow_mapping_key');
+            return $self->process_empty_scalar($token->end_mark);
+        }
+    }
+    else {
+        $self->state('parse_flow_mapping_key');
+        my $token = $self->scanner->peek_token();
+        return $self->process_empty_scalar($token->start_mark);
+    }
 }
 
 sub parse_flow_mapping_empty_value {
     my $self = shift;
-    die "parse_flow_mapping_empty_value";
+    $self->state('parse_flow_mapping_key');
+    return $self->process_empty_scalar($self->scanner->peek_token()->start_mark);
 }
 
 sub process_empty_scalar {
@@ -601,7 +832,7 @@ sub process_empty_scalar {
     return YAML::Perl::Event::Scalar->new(
         anchor     => undef,
         tag        => undef,
-        implicit   => 1,     # what does (True, False) mean??
+        implicit   => [True, False],
         value      => '',
         start_mark => $mark,
         end_mark   => $mark
